package org.apache.maven.jalopy;

/* ====================================================================
 *   Licensed to the Apache Software Foundation (ASF) under one or more
 *   contributor license agreements.  See the NOTICE file distributed with
 *   this work for additional information regarding copyright ownership.
 *   The ASF licenses this file to You under the Apache License, Version 2.0
 *   (the "License"); you may not use this file except in compliance with
 *   the License.  You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 * ====================================================================
 */

import de.hunsicker.jalopy.Jalopy;
import de.hunsicker.jalopy.storage.Convention;
import de.hunsicker.jalopy.storage.History;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.codehaus.plexus.util.DirectoryScanner;
import org.codehaus.plexus.util.StringUtils;


/**
 * A bean to format all source files according to code conventions.
 *
 * @author jruiz@exist.com
 * @author <a href="mailto:evenisse@codehaus.org">Emmanuel Venisse</a>
 * @author <a href="mailto:ltheussl@apache.org">Lukas Theussl</a>
 * @version $Id: JalopyBean.java 531612 2007-04-23 21:28:38Z ltheussl $
 */

public class JalopyBean
{
    /** Log. */
    private static final Log LOG = LogFactory.getLog( JalopyBean.class );

    /**
     * Sets the file format of the output files.
     * The file format controls what end of line character is used.
     * Either one of "UNIX", "DOS", "MAC", "DEFAULT" or "AUTO" can be used.
     */
    private String fileFormat;

    /**
     * Specifies the history policy to use.
     * Either one of "COMMENT", "FILE" or "NONE" can be used.
     */
    private String historyPolicy;

    /**
     * Indicates whether a run should be held if errors occured.
     */
    private boolean failOnError;

    /**
     * Indicates whether to enable the code inspector.
     */
    private boolean inspect;

    /**
     * Indicates whether keep backups of formatted source files.
     */
    private boolean backup;

    /**
     * Indicates whether to force formatting of source files.
     */
    private boolean force;

    /**
     * The source directory.
     */
    private File sourceDirectory;

    /**
     *  The test source directory.
     */
    private File testSourceDirectory;

    /**
     * Sets the preferences file to use - given either relative to
     * the project's basedir or as an absolute local path or internet address.
     * If omitted, the current preferences are used, if available.
     * Otherwise the Jalopy build-in defaults will be used.
     */
    private File convention;

    /**
     * For Source Directory. Specifies a fileset source file to format.
     * This is a comma- or space-separated list of patterns of files.
     */
    private String srcIncludesPattern;

    /**
     * For Source Directory. Source files excluded from format.
     * This is a comma- or space-separated list of patterns of files.
     */
    private String srcExcludesPattern;

    /**
     * For Test Directory. Specifies a fileset source file to format.
     * This is a comma- or space-separated list of patterns of files.
     */
    private String testIncludesPattern;

    /**
     * For Test Directory. Source files excluded from format.
     * This is a comma- or space-separated list of patterns of files.
     */
    private String testExcludesPattern;

    /**
     * The main method: starts formatting of source files.
     *
     * @throws Exception Exception
     */
    public void execute() throws Exception
    {

        LOG.info("[jalopy] Jalopy Java Source Code Formatter " + Jalopy.getVersion());

        try
        {
            if ( getSourceDirectory().exists() )
            {
                String[] filesToFormat = getIncludedFiles( getSourceDirectory(), getSrcIncludesPattern(),
                                                           getSrcExcludesPattern() );

                LOG.info( "[jalopy] Format " + filesToFormat.length + " source files in " + getSourceDirectory().toString() );
                formatFiles( getSourceDirectory(), filesToFormat );
                LOG.info( "[jalopy] " + filesToFormat.length + " source files formatted" );
            }

            if ( getTestSourceDirectory().exists() )
            {
                String[] filesToFormat = getIncludedFiles( getTestSourceDirectory(), getTestIncludesPattern(),
                                                           getTestExcludesPattern() );

                LOG.info( "[jalopy] Format " + filesToFormat.length + " source files in " + getTestSourceDirectory().toString() );
                formatFiles( getTestSourceDirectory(), filesToFormat );
                LOG.info( "[jalopy] " + filesToFormat.length + " source files formatted" );
            }
        }
        catch ( Exception e )
        {
            e.printStackTrace();
        }
    }

    /**
     * Returns a string array of files to format.
     *
     * @param directory The base directory
     * @param includes The includes pattern
     * @param excludes The excludes pattern
     * @return String[]
     */
    private String[] getIncludedFiles( File directory, String includes, String excludes )
    {
        DirectoryScanner scanner = new DirectoryScanner();

        scanner.setBasedir( directory );

        scanner.setIncludes( StringUtils.split( includes, "," ) );

        scanner.setExcludes( StringUtils.split( excludes, "," ) );

        scanner.scan();

        String[] filesToFormat = scanner.getIncludedFiles();

        return filesToFormat;
    }

    /**
     * Formats the source files.
     *
     * @param directory The base directory
     * @param filesToFormat A string array of files to format
     * @throws FileNotFoundException FileNotFoundException
     * @throws Exception Exception
     */
    private void formatFiles( File directory, String[] filesToFormat )
        throws FileNotFoundException, Exception
    {
        Jalopy jalopy = createJalopy();

        for ( int i = 0; i < filesToFormat.length; i++ )
        {
            File currentFile = new File( directory, filesToFormat[i] );

            jalopy.setInput( currentFile );

            jalopy.setOutput( currentFile );

            if ( isBackup() )
            {
                jalopy.setBackupDirectory( currentFile.getParentFile() );
            }

            jalopy.format();

            logMessage( jalopy, currentFile );
        }
    }

    /**
     * Logs a message depending on the jalopy exit status.
     *
     * @param jalopy The jalopy instance
     * @param currentFile The file whose status is to be logged
     * @throws Exception Exception
     */
    private void logMessage( Jalopy jalopy, File currentFile )
        throws Exception
    {

        if ( jalopy.getState() == Jalopy.State.OK )
        {
            LOG.debug( "[jalopy] " + currentFile + " formatted correctly." );
        }
        else if ( jalopy.getState() == Jalopy.State.WARN )
        {
            LOG.warn( "[jalopy] " + currentFile + " formatted with warnings." );
        }
        else if ( jalopy.getState() == Jalopy.State.ERROR )
        {
            LOG.error( "[jalopy] " + currentFile + " could not be formatted." );

            if ( isFailOnError() )
            {
                throw new Exception( currentFile + " could not be formatted." );
            }
        }
        else
        {
            LOG.warn( "[jalopy] " + currentFile + " formatted with unknown state." );
        }
    }

    /**
     * Creates a new instance of a Jalopy formatter with all bean variables being set.
     *
     * @return Jalopy
     */
    private Jalopy createJalopy()
    {
        Jalopy jalopy = new Jalopy();

        try
        {
            if ( ( convention != null ) && convention.exists() )
            {
                LOG.debug( "[jalopy] Using convention file : " + convention.getAbsoluteFile() );
                Jalopy.setConvention( convention );
            }
            else
            {
                LOG.warn( "[jalopy] Convention file does not exist: " + convention.getAbsoluteFile() );
                LOG.warn( "[jalopy] Using default convention : jalopy.xml");
                InputStream in = this.getClass().getClassLoader().getResourceAsStream( "plugin-resources/jalopy.xml" );
                Convention.importSettings( in, Convention.EXTENSION_XML );
            }
        }
        catch ( IOException ex )
        {
            ex.printStackTrace();
        }

        jalopy.setFileFormat( getFileFormat() );

        jalopy.setInspect( isInspect() );

        if ( !getHistoryPolicy().equalsIgnoreCase( "none" ) )
        {
            jalopy.setHistoryPolicy( History.Policy.valueOf( getHistoryPolicy() ) );
        }

        // TODO: what are the allowed values for historyMethod?
        //Convention settings = Convention.getInstance();
        //History.Method historyMethod = History.Method.valueOf( settings.get( ConventionKeys.HISTORY_METHOD,
        //                                                                     ConventionDefaults.HISTORY_METHOD ) );
        //jalopy.setHistoryMethod( historyMethod );

        jalopy.setBackup( isBackup() );

        jalopy.setForce( isForce() );

        return jalopy;
    }

    /**
     * Returns the fileFormat property.
     *
     * @return String
     */
    public String getFileFormat()
    {
        return fileFormat;
    }

    /**
     * Sets the fileFormat property.
     *
     * @param newfileFormat The new property
     */
    public void setFileFormat( String newfileFormat )
    {
        this.fileFormat = newfileFormat;
    }

    /**
     * Returns the failOnError property.
     *
     * @return boolean
     */
    public boolean isFailOnError()
    {
        return failOnError;
    }

    /**
     * Sets the failOnError property.
     *
     * @param newfailOnError The new property
     */
    public void setFailOnError( boolean newfailOnError )
    {
        this.failOnError = newfailOnError;
    }

    /**
     * Returns the inspect property.
     *
     * @return boolean
     */
    public boolean isInspect()
    {
        return inspect;
    }

    /**
     * Sets the inspect property.
     *
     * @param newinspect The new property
     */
    public void setInspect( boolean newinspect )
    {
        this.inspect = newinspect;
    }

    /**
     * Returns the backup property.
     *
     * @return boolean
     */
    public boolean isBackup()
    {
        return backup;
    }

    /**
     * Sets the backup property.
     *
     * @param newbackup The new property
     */
    public void setBackup( boolean newbackup )
    {
        this.backup = newbackup;
    }

    /**
     * Returns the force property.
     *
     * @return boolean
     */
    public boolean isForce()
    {
        return force;
    }

    /**
     * Sets the force property.
     *
     * @param newforce The new property
     */
    public void setForce( boolean newforce )
    {
        this.force = newforce;
    }

    /**
     * Returns the sourceDirectory property.
     *
     * @return File
     */
    public File getSourceDirectory()
    {
        return sourceDirectory;
    }

    /**
     * Sets the sourceDirectory property.
     *
     * @param newsourceDirectory The new property
     */
    public void setSourceDirectory( File newsourceDirectory )
    {
        this.sourceDirectory = newsourceDirectory;
    }

    /**
     * Returns the srcIncludesPattern property.
     *
     * @return String
     */
    public String getSrcIncludesPattern()
    {
        return srcIncludesPattern;
    }

    /**
     * Sets the srcIncludesPattern property.
     *
     * @param newsrcIncludesPattern The new property
     */
    public void setSrcIncludesPattern( String newsrcIncludesPattern )
    {
        this.srcIncludesPattern = newsrcIncludesPattern;
    }

    /**
     * Returns the srcExcludesPattern property.
     *
     * @return String
     */
    public String getSrcExcludesPattern()
    {
        return srcExcludesPattern;
    }

    /**
     * Sets the srcExcludesPattern property.
     *
     * @param newsrcExcludesPattern The new property
     */
    public void setSrcExcludesPattern( String newsrcExcludesPattern )
    {
        this.srcExcludesPattern = newsrcExcludesPattern;
    }

    /**
     * Returns the historyPolicy property.
     *
     * @return String
     */
    public String getHistoryPolicy()
    {
        return historyPolicy;
    }

    /**
     * Sets the historyPolicy property.
     *
     * @param newhistoryPolicy The new property
     */
    public void setHistoryPolicy( String newhistoryPolicy )
    {
        this.historyPolicy = newhistoryPolicy;
    }

    /**
     * Returns the convention property.
     *
     * @return File
     */
    public File getConvention()
    {
        return convention;
    }

    /**
     * Sets the convention property.
     *
     * @param newconvention The new property
     */
    public void setConvention( File newconvention )
    {
        this.convention = newconvention;
    }

    /**
     * Returns the testSourceDirectory property.
     *
     * @return File
     */
    public File getTestSourceDirectory()
    {
        return testSourceDirectory;
    }

    /**
     * Sets the testSourceDirectory property.
     *
     * @param newtestSourceDirectory The new property
     */
    public void setTestSourceDirectory( File newtestSourceDirectory )
    {
        this.testSourceDirectory = newtestSourceDirectory;
    }

    /**
     * Returns the testIncludesPattern property.
     *
     * @return String
     */
    public String getTestIncludesPattern()
    {
        return testIncludesPattern;
    }

    /**
     * Sets the testIncludesPattern property.
     *
     * @param newtestIncludesPattern The new property
     */
    public void setTestIncludesPattern( String newtestIncludesPattern )
    {
        this.testIncludesPattern = newtestIncludesPattern;
    }

    /**
     * Returns the testExcludesPattern property.
     *
     * @return String
     */
    public String getTestExcludesPattern()
    {
        return testExcludesPattern;
    }

    /**
     * Sets the testExcludesPattern property.
     *
     * @param newtestExcludesPattern The new property
     */
    public void setTestExcludesPattern( String newtestExcludesPattern )
    {
        this.testExcludesPattern = newtestExcludesPattern;
    }
}
